<?php
/**
 * Elementor views manager
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Jet_Engine_Blocks_Views_Render' ) ) {

	/**
	 * Define Jet_Engine_Blocks_Views_Render class
	 */
	class Jet_Engine_Blocks_Views_Render {

		private $contents = array();
		private $enqueued_css = array();
		private $printed_css = array();
		private $current_listing = null;

		/**
		 * Counter of wp containers in listing item.
		 * @var int
		 */
		private $counter = 0;

		/**
		 * Current listing css generated by wp.
		 * @var string
		 */
		private $wp_listing_css = null;

		/**
		 * Printed listing css generated by wp.
		 * @var array
		 */
		private $printed_wp_css = array();

		public function __construct() {
			add_action( 'enqueue_block_assets', array( jet_engine()->frontend, 'frontend_styles' ) );

			add_action( 'wp_footer', array( $this, 'print_css' ) );
			add_action( 'jet-engine/listing/grid/after', array( $this, 'print_preview_css' ) );

			add_action( 'jet-engine/blocks-views/print-template-styles', array( $this, 'print_template_css' ) );
		}

		/**
		 * Print preview CSS
		 *
		 * @return [type] [description]
		 */
		public function print_preview_css() {
			$this->print_css();
		}

		public function print_css() {
			foreach ( $this->enqueued_css as $post_id => $css ) {
				if ( ! empty( $css ) && ! in_array( $post_id, $this->printed_css ) ) {
					echo $css;
					$this->printed_css[] = $post_id;
				}
			}
		}

		/**
		 * Returns listing content for given listing ID
		 *
		 * @return [type] [description]
		 */
		public function get_listing_content( $listing_id ) {
			$content = $this->get_raw_content( $listing_id );
			$this->enqueue_listing_css( $listing_id );
			return do_shortcode( $this->parse_content( $content, $listing_id ) );
		}

		public function fix_context( $context ) {

			$object = jet_engine()->listings->data->get_current_object();

			if ( $object && 'WP_Post' === get_class( $object ) ) {
				$context['postId']   = $object->ID;
				$context['postType'] = $object->post_type;
			}

			return $context;

		}

		/**
		 * Returns current listing ID
		 *
		 * @return [type] [description]
		 */
		public function get_current_listing_id() {
			return $this->current_listing;
		}

		/**
		 * Parse listing item content
		 *
		 * @param  [type] $content [description]
		 * @return [type]          [description]
		 */
		public function parse_content( $content, $listing_id ) {

			add_filter( 'render_block_context', array( $this, 'fix_context' ) );

			// Removed `wp_render_layout_support_flag` filter and added modified filter.
			remove_filter( 'render_block', 'wp_render_layout_support_flag' );
			add_filter( 'render_block', array( $this, 'wp_render_layout_support_flag' ), 10, 2 );

			$this->current_listing = $listing_id;
			$parsed = do_blocks( $content );
			$this->current_listing = false;

			remove_filter( 'render_block_context', array( $this, 'fix_context' ) );

			// Restore `wp_render_layout_support_flag` filter.
			$this->counter = 0;
			add_filter( 'render_block', 'wp_render_layout_support_flag', 10, 2 );
			remove_filter( 'render_block', array( $this, 'wp_render_layout_support_flag' ) );

			// Enqueue wp listing css.
			$inline_wp_css = $this->enqueue_wp_listing_css( $listing_id );

			if ( ! empty( $inline_wp_css ) ) {
				$parsed = $inline_wp_css . $parsed;
			}

			return $parsed;

		}

		public function print_template_css( $template_id ) {
			$this->enqueue_listing_css( $template_id, true );
		}

		public function enqueue_listing_css( $listing_id, $print = false ) {

			if ( isset( $this->enqueued_css[ $listing_id ] ) ) {
				return;
			}

			$css    = get_post_meta( $listing_id, '_jet_engine_listing_css', true );
			$result = '';
			$style  = '';

			if ( class_exists( '\JET_SM\Gutenberg\Style_Manager' ) ) {
				$style = \JET_SM\Gutenberg\Style_Manager::get_instance()->get_blocks_style( $listing_id );
			}
			
			$css .= $style;

			if ( $css ) {
				$css    = str_replace( 'selector', '.jet-listing-grid--' . $listing_id, $css );
				$result = '<style class="listing-css-' . $listing_id . '">' . $css . '</style>';
			}

			if ( $print ) {
				echo $result;
				$this->printed_css[] = $listing_id;
			} else {
				$this->enqueued_css[ $listing_id ] = $result;
			}

			if ( class_exists( '\JET_SM\Gutenberg\Style_Manager' ) ) {
				\JET_SM\Gutenberg\Style_Manager::get_instance()->render_blocks_fonts( $listing_id );
			}

		}



		/**
		 * Returns raw listing content
		 *
		 * @param  [type] $listing_id [description]
		 * @return [type]             [description]
		 */
		public function get_raw_content( $listing_id ) {

			if ( ! isset( $this->contents[ $listing_id ] ) ) {
				$post = get_post( $listing_id );
				$this->contents[ $listing_id ] = $post->post_content;
			}

			return $this->contents[ $listing_id ];
		}

		/**
		 * `wp_render_layout_support_flag` is rewritten to prevent conflict of not uniq css classes (`wp-container-`) on ajax.
		 * See: https://github.com/Crocoblock/issues-tracker/issues/700
		 *
		 * @param $block_content
		 * @param $block
		 *
		 * @return string|string[]|null
		 */
		public function wp_render_layout_support_flag( $block_content, $block ) {
			$block_type     = WP_Block_Type_Registry::get_instance()->get_registered( $block['blockName'] );
			$support_layout = block_has_support( $block_type, array( '__experimentalLayout' ), false );

			if ( ! $support_layout ) {
				return $block_content;
			}

			$block_gap             = wp_get_global_settings( array( 'spacing', 'blockGap' ) );
			$default_layout        = wp_get_global_settings( array( 'layout' ) );
			$has_block_gap_support = isset( $block_gap ) ? null !== $block_gap : false;
			$default_block_layout  = _wp_array_get( $block_type->supports, array( '__experimentalLayout', 'default' ), array() );
			$used_layout           = isset( $block['attrs']['layout'] ) ? $block['attrs']['layout'] : $default_block_layout;
			if ( isset( $used_layout['inherit'] ) && $used_layout['inherit'] ) {
				if ( ! $default_layout ) {
					return $block_content;
				}
				$used_layout = $default_layout;
			}

			$class_names     = array();
			// Editing $container_class var
			// $container_class = wp_unique_id( 'wp-container-' );
			$container_class = sprintf( 'wp-container-%s-%s', $this->current_listing, ++$this->counter );
			$class_names[]   = $container_class;

			// The following section was added to reintroduce a small set of layout classnames that were
			// removed in the 5.9 release (https://github.com/WordPress/gutenberg/issues/38719). It is
			// not intended to provide an extended set of classes to match all block layout attributes
			// here.
			if ( ! empty( $block['attrs']['layout']['orientation'] ) ) {
				$class_names[] = 'is-' . sanitize_title( $block['attrs']['layout']['orientation'] );
			}

			if ( ! empty( $block['attrs']['layout']['justifyContent'] ) ) {
				$class_names[] = 'is-content-justification-' . sanitize_title( $block['attrs']['layout']['justifyContent'] );
			}

			if ( ! empty( $block['attrs']['layout']['flexWrap'] ) && 'nowrap' === $block['attrs']['layout']['flexWrap'] ) {
				$class_names[] = 'is-nowrap';
			}

			$gap_value = _wp_array_get( $block, array( 'attrs', 'style', 'spacing', 'blockGap' ) );
			// Skip if gap value contains unsupported characters.
			// Regex for CSS value borrowed from `safecss_filter_attr`, and used here
			// because we only want to match against the value, not the CSS attribute.
			if ( is_array( $gap_value ) ) {
				foreach ( $gap_value as $key => $value ) {
					$gap_value[ $key ] = $value && preg_match( '%[\\\(&=}]|/\*%', $value ) ? null : $value;
				}
			} else {
				$gap_value = $gap_value && preg_match( '%[\\\(&=}]|/\*%', $gap_value ) ? null : $gap_value;
			}

			$fallback_gap_value = _wp_array_get( $block_type->supports, array( 'spacing', 'blockGap', '__experimentalDefault' ), '0.5em' );

			// If a block's block.json skips serialization for spacing or spacing.blockGap,
			// don't apply the user-defined value to the styles.
			$should_skip_gap_serialization = wp_should_skip_block_supports_serialization( $block_type, 'spacing', 'blockGap' );
			$style                         = wp_get_layout_style( ".$container_class", $used_layout, $has_block_gap_support, $gap_value, $should_skip_gap_serialization, $fallback_gap_value );
			// This assumes the hook only applies to blocks with a single wrapper.
			// I think this is a reasonable limitation for that particular hook.
			$content = preg_replace(
				'/' . preg_quote( 'class="', '/' ) . '/',
				'class="' . esc_attr( implode( ' ', $class_names ) ) . ' ',
				$block_content,
				1
			);

			// Style stored to `wp_listing_css` prop.
			// wp_enqueue_block_support_styles( $style );
			$this->wp_listing_css .= $style;

			return $content;
		}

		public function enqueue_wp_listing_css( $listing_id ) {

			$inline_css = null;

			if ( ! empty( $this->wp_listing_css ) && ! in_array( $listing_id, $this->printed_wp_css ) ) {

				if ( wp_doing_ajax() ) {
					$inline_css = sprintf( '<style>%s</style>', $this->wp_listing_css );
				} else {
					wp_enqueue_block_support_styles( $this->wp_listing_css );
				}

				$this->printed_wp_css[] = $listing_id;
			}

			$this->wp_listing_css = null;

			return $inline_css;
		}

	}

}
